import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import React, { useCallback, useContext, useEffect, useMemo, useRef, useState } from 'react';
import { makeCancelablePromise } from '@billogram/cancelable-promise';
import dataFetched from './dataFetched';
import dataFetchedError from './dataFetchedError';
import dataUpdate from './dataUpdate';

function noop() {} // eslint-disable-next-line @typescript-eslint/no-explicit-any


var FoundationDataProviderContext = React.createContext({
  fetch: function () {
    var _fetch = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee() {
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    function fetch() {
      return _fetch.apply(this, arguments);
    }

    return fetch;
  }(),
  result: {
    state: 'fetching-initial'
  },
  update: noop
});
export function FoundationDataProvider(_ref) {
  var children = _ref.children,
      fetchData = _ref.fetchData,
      initialFetchDataOptions = _ref.initialFetchDataOptions;
  var cancel = useRef(noop);

  var _useState = useState({
    state: 'fetching-initial'
  }),
      _useState2 = _slicedToArray(_useState, 2),
      result = _useState2[0],
      setResult = _useState2[1];

  var fetch = useCallback(
  /*#__PURE__*/
  function () {
    var _fetch2 = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee2(options) {
      var _makeCancelablePromis, _makeCancelablePromis2, newRequest, newCancel, token, _token, errors;

      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.prev = 0;
              cancel.current();
              setResult(function (result) {
                switch (result.state) {
                  case 'error-initial':
                  case 'fetching-initial':
                    {
                      return {
                        state: 'fetching-initial'
                      };
                    }

                  default:
                    {
                      return {
                        data: result.data,
                        state: 'fetching'
                      };
                    }
                }
              });
              _makeCancelablePromis = makeCancelablePromise(fetchData(options)), _makeCancelablePromis2 = _slicedToArray(_makeCancelablePromis, 2), newRequest = _makeCancelablePromis2[0], newCancel = _makeCancelablePromis2[1];
              cancel.current = newCancel;
              _context2.next = 7;
              return newRequest;

            case 7:
              token = _context2.sent;

              if (!token.isPromiseCanceled) {
                setResult(dataFetched(token.value));
              }

              _context2.next = 15;
              break;

            case 11:
              _context2.prev = 11;
              _context2.t0 = _context2["catch"](0);
              _token = _context2.t0;

              if (!_token.isPromiseCanceled) {
                errors = _token.reason !== void 0 && _token.reason.data !== void 0 ? _token.reason.data : {};
                setResult(dataFetchedError(errors));
              }

            case 15:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, null, [[0, 11]]);
    }));

    function fetch(_x) {
      return _fetch2.apply(this, arguments);
    }

    return fetch;
  }(), [fetchData]);
  var update = useCallback(function update(data) {
    setResult(dataUpdate(data));
  }, []);
  useEffect(function () {
    fetch(initialFetchDataOptions);
    return function () {
      cancel.current();
      cancel.current = noop;
    };
  }, [fetch, initialFetchDataOptions]);
  var context = useMemo(function () {
    return {
      fetch: fetch,
      result: result,
      update: update
    };
  }, [fetch, result, update]);
  return React.createElement(FoundationDataProviderContext.Provider, {
    value: context
  }, children);
}
export function FoundationDataConsumer(_ref2) {
  var children = _ref2.children;
  return React.createElement(FoundationDataProviderContext.Consumer, null, function (context) {
    return children(context);
  });
}
export function useDataProvider() {
  return useContext(FoundationDataProviderContext);
}
//# sourceMappingURL=FoundationDataProviderContext.js.map